# simbiofilm

This readme is very much WIP, please contact me if you would like more detail!


## Quickstart

1. Install [miniconda](https://docs.conda.io/en/latest/miniconda.html)
2. `conda create -c conda-forge -n PROJECTNAME numpy scipy numba fipy scipy pyamg libgcc`
3. `conda activate PROJECTNAME`
4. `pip install git+https://github.com/scikit-fmm/scikit-fmm.git@ff8a4673b0b1ead9a725762eed99bb45c0e50ec4`
5. `git clone git@github.com:simbiofilm/simbiofilm.git`
6. `pip install -e simbiofilm`
7. `python simbiofilm/examples/simple.py`


## The run script

The simbiofilm run script generally has two components: the default
configuration setup, and the simulation setup. The simulation setup, outside
of a few options, generally doesn't change within a single study, and should
remain relatively constant. The configuration is varies the parameters for the
study and any particular models you are testing.

The configuration in the script is a dictionary essentially a dictionary
with a method that allows '.' access. For example, `cfg.general.seed` is
allowed, rather than only allowing `cfg['general']['seed']`.

This dictionary gets potentially modified by the `-p` command line option,
which has the format
`python runscript.py -p sectionname1:paramname1,sectionname2:paramname2 value1,value2`
For example,
`python runscript.py -p general:seed,species1:density 4321,210e3`

Note that the `sb.cfg_from_dict` will try its best to convert values to their
appropriate types, but sometimes will need help. In particular, don't try to use
lists/other iterables, and ints might need explicit conversion.

See the examples for the structure of the run scripts, but they have 3 major components:

1. The configuration, which is basically a dict wrapped by the `cfg_from_dict` function
2. The simulation setup, which creates the necessary simbiofilm simulation objects:
  a. space
  b. simulation
  c. containers (solutes and biomass/particulate containers)
  d. behaviors (interactions between containers)
3. A command line option parser, so that the parameter parsing above works correctly.

See examples/simple.py for a fairly reduced example.

# Multiple simulations

In order to run a large number of simulations, there are three basic steps:

1. Create a list of parameters
2. Create a job that will run those simulations
3. Run the job

## Make parameters

To create a list of parameters, we use the `Paramgroup` object from the
parametergroup.py module. To use, create a python file with the following
contents:

``` python
from parametergroup import Paramgroup
pg = Paramgroup()

 # set some defaults
pg.default('general:runtime', 20)
pg.default('substrate:max', 5)
pg.default('species1:mu', 24.5)
pg.default('species2:mu', 21.0)
 # Note that defaults can also have ranges, like below.

 # add a group and the combinations
pg.add('test_mu', 'species1:mu', [20, 22, 25])
pg.add('test_mu', 'species2:mu', [20, 22, 25])
 # This will run each combination: [20, 20], [22, 20], [25, 20], [20, 22], etc

 # add a group and the combinations
pg.add('test_smax', 'species1:mu', 20)  # changes the 'default' for this group
pg.add('test_smax', 'species2:mu', 22)
pg.add('test_smax', 'substrate:max', [1, 3, 6, 9])

pg.set_repeats(6)  # run each combination 6 times

 # Shows the total number, here: 6 * (3 * 3 + 1 * 1 * 4) = 78
print(f'number of runs to do: {len(pg)}')

pg.print('test_output.csv')

```


## Jobs

A 'job' in the nomenclature here will just be a list of simulations and
associated files that are prepared to run a list of simulations. The included
job scripts work for LSF and torque clusters (`bsub` and `qsub`, respectively),
and also at the command line.

The script is designed to run independently from other instances of the same
script, which allows the user to simply run the script for as many times as
the number of threads they have available for computation.

For example, if I have a 16 core computer and want to use 10 cores to run
simulations, I would use `parallel "bash name.job" ::: ${seq 10}.

To understand the job script, imagine there is a stack of cards, where each
card details a single simulation. There are 10 people who want the next card,
who gets to access it? The first one to call 'dibs' gets the top card, while
the others wait. When they draw their card, they let everyone know they are
done drawing, and let the group call dibs again and draw the next card.

This way, there are no repeated simulations, AND the script itself doesn't
change with the number of threads running the simulations.

If you use LSF or torque to run the simulations, edit the comments at the top
of the file as necessary, then a simple job submission should read it
appropriately. Use the parmeters detailed in `make_job.sh -h` if you need to
specify wall time, keeping in mind the expected wall time should be approximately:
`time per simulation * number of simulations / size of job array` per job.

** NOTE you will have to add the conda environment name to the template files!
